@@ -1155,13 +1155,13 @@ SEXP emML2(NumericVector y, NumericMatrix X1, NumericMatrix X2,
 
 // [[Rcpp::export]]
 SEXP mrr(NumericMatrix Y, NumericMatrix X){
+  // Convergence parameters
   int maxit = 200; double tol = 10e-8;
   // Obtain environment containing function
   Rcpp::Environment base("package:base");
   Rcpp::Function solve = base["solve"];
   // Functions starts here
   int k = Y.ncol(), p = X.ncol(), n0 = X.nrow();
-  // Handle missings Y's
   NumericMatrix fit(n0,k),o(n0,k),y(n0,k),e(n0,k);
   for(int i=0; i<k; i++){
     o(_,i) = ifelse(is_na(Y(_,i)),0,1);
@@ -1175,9 +1175,10 @@ SEXP mrr(NumericMatrix Y, NumericMatrix X){
      xx(i,j) = sum(X(_,i)*X(_,i)*o(_,j));
       tmp = sum(X(_,i)*o(_,j))/n(j);
       vx(i,j) = xx(i,j)/n(j)-tmp*tmp;}}
+  //NumericVector MSx = colSums(xx);
   NumericVector MSx = colSums(vx);
   // Beta, intersept and residuals
-  NumericMatrix b(p,k),vb(k,k),iG(k,k),LHS(k,k);
+  NumericMatrix b(p,k),vb(k,k),iG(k,k),rho(k,k),LHS(k,k);
   NumericVector b0(k),b1(k),eM(k),mu(k),vy(k),ve(k),RHS(k);
   mu = colSums(y)/n;
   for(int i=0; i<k; i++){for(int j=0; j<k; j++){vb(i,j) = 0;}}
@@ -1185,11 +1186,13 @@ SEXP mrr(NumericMatrix Y, NumericMatrix X){
     e(_,i) = (y(_,i)-mu(i))*o(_,i);
     vy(i) = sum(e(_,i)*e(_,i))/(n(i)-1);
     ve(i) = vy(i)*0.5;
-    vb(i,i) = ve(i)/MSx(i);}
+    vb(i,i) = ve(i)/MSx(i);
+    rho(i,i) = 1;}
   iG = solve(vb);
   // Convergence control
   NumericMatrix bc(p,k);
-  int numit = 0; double cnv = 1;
+  int numit = 0;
+  double cnv = 1;
   // Loop
   while(numit<maxit){
     // Gauss-Seidel loop
@@ -1211,15 +1214,18 @@ SEXP mrr(NumericMatrix Y, NumericMatrix X){
     mu = mu+eM;
     for(int j=0; j<k; j++){e(_,j) = (e(_,j)-eM(j))*o(_,j);}
     // Variance components update
-    for(int i=0; i<k; i++){for(int j=0; j<k; j++){vb(i,j) = sum(b(_,i)*b(_,j))/(p-1);}}
-    for(int i=0; i<k; i++){ve(i) = sum(e(_,i)*y(_,i))/(n(i)-1);vb(i,i) = (1.001*vy(i)-ve(i))/MSx(i);}
+    for(int i=0; i<k; i++){ ve(i) = sum(e(_,i)*y(_,i))/(n(i)-1);}
+    for(int i=0; i<n0; i++){ for(int j=0; j<k; j++){ fit(i,j) = sum(X(i,_)*b(_,j));}}
+    for(int i=0; i<k; i++){ for(int j=0; j<k; j++){
+      vb(i,j) = (sum(fit(_,i)*y(_,j))+sum(fit(_,j)*y(_,i))) / ((n(i)*MSx(i))+(n(j)*MSx(j))) ;}}
+    for(int i=0; i<k; i++){vb(i,i)=vb(i,i)*1.01;} // Ridging
     iG = solve(vb);
     // Convergence
     ++numit;
     cnv = sum(abs(bc-b));
     if( cnv<tol ){break;}}
-  // Fitting the model and heritability
-  NumericVector h2(k);
+  // Fitting the model
+  NumericVector h2(k); 
   for(int i=0; i<n0; i++){for(int j=0; j<k; j++){fit(i,j) = sum(X(i,_)*b(_,j))+mu(j);}}
   for(int i=0; i<k; i++){ h2 = (vb(i,i)*MSx(i))/((vb(i,i)*MSx(i))+ve); }
   // Output
